module Scoreable
  extend ActiveSupport::Concern

  def normalized_views
    average_by_max(views.count, self.class.maximum(:views_count))
  end

  def normalized_likes
    average_by_max(likes.count, self.class.maximum(:likes_count))
  end

  def normalized_downloads
    average_by_max(downloads.count, self.class.maximum(:downloads_count))
  end

  def normalized_shares
    average_by_max(shares.count, self.class.maximum(:shares_count))
  end

  def normalized_bookmarked
    max = Bookmark.where(bookmarkable_type: self.class.name).group(:bookmarkable_id).order('count_all DESC').count
    value = Bookmark.where(bookmarkable_id: id, bookmarkable_type: self.class.name).count

    return 0.0 if max.blank?

    average_by_max(value, max.first[1])
  end

  private

  def average_by_max(value, max)
    return 0.0 if max <= 0.0
    value.to_f / max.to_f
  end

end
