class ScoreCalculatorService

  def initialize(object)
    @object = object
    @class = @object.class.name
    @value = nil
  end

  def calculate(values = nil)
    values = object_hash if values.nil?
    values = values.with_indifferent_access

    sum_average = @object.is_a?(User) ? true : false

    sum = 0

    Score.object_type(@object.class.name).find_each do |score|
      if sum_average
        max = score.max_value.to_f

        next if max == 0

        sum += (values[score.code].to_f / max) * score.weight.to_f
      else
        sum += values[score.code].to_f * score.weight.to_f
      end
    end

    @value = (sum.to_f / sum_weights.to_f).round(6)
  end

  def category_by_reputation
    calculate if @value.nil?

    UserCategory.order(:reference).each do |uc|
      next if @value > uc.reference
      return uc.id
    end
  end

  private

  def learning_object_hash
    {
      'score_average_reviews': @object,
      'score_thumbnail': @object.thumbnail.nil? ? 0 : 1,
      'score_views': @object.normalized_views,
      'score_likes': @object.normalized_likes,
      'score_downloads': @object.normalized_downloads,
      'score_shares': @object.normalized_shares,
      'score_description': @object.description.nil? ? 0 : 1,
      'score_user_category': @object,
      'score_added_to_collections': @object.normalized_collected,
      'score_added_to_bookmarks': @object.normalized_bookmarked
    }
  end

  def object_hash
    Score.object_type(@object.class.name).map { |s| [s.code, @object.try(s.code)] }.to_h
  end

  def sum_weights
    Score.object_type(@object.class.name).pluck(:weight).inject(0.0) { |a, e| a + e }
  end

end
